

-- Helpers
--

function oneshot_timer(func, time)
  local wrap_func = nil
  wrap_func = function()
    func()
    renoise.tool():remove_timer(wrap_func)
  end
  if not renoise.tool():has_timer(func) then
    renoise.tool():add_timer(wrap_func, time)
  end
end

-- switch function for case-based execution
function switch(case)
  return function(cases)
           local val = cases[case] or cases["default"]
           return type(val) == "function" and val() or val
         end
end


-- Classes
--

class 'App'

  function App:__init(args)
    self.song = args.song
    self.AppData = AppData { app = self }
    
    -- This will "break" if not wrapping a timer
    -- The non-wrapped oprint gives nil when inserting a new pattern at the very end
    self.song.selected_pattern_index_observable:add_notifier(
      function()
        oprint(self.AppData.patterns:property(self.song.selected_pattern_index))
        oneshot_timer(function() oprint(self.AppData.patterns:property(self.song.selected_pattern_index)) end, 50)
      end
    )
    
  end


class 'Pattern' (renoise.Document.DocumentNode)

  function Pattern:__init(args)
    renoise.Document.DocumentNode.__init(self)

    self:add_property("Modules", renoise.Document.DocumentList())
  end


class 'AppData'

  function AppData:__init(args)
    self.app = args.app
    self.main_doc = self:init_internal_patterns()

    self.app.song.patterns_observable:add_notifier(AppData.update_internal_patterns, self)
  end
  
  AppData.patterns = property(function(obj) return obj.main_doc:property("Patterns") end)  

  -- initialise the pattern list (or eventually load it from tool_data)
  function AppData:init_internal_patterns()
    local main_doc = renoise.Document.create("Arranger") {
        Patterns = renoise.Document.DocumentList()
    }

    for pat_idx, pat in ipairs(self.app.song.patterns) do
      main_doc:property("Patterns"):insert( Pattern() )
    end
    
    return main_doc
  end

  function AppData:update_internal_patterns(event)

    switch(event.type) {
      insert = function() self.patterns:insert(event.index, Pattern()) end,
      remove = function() self.patterns:remove(event.index) end,
        swap = function() self.patterns:swap(event.index1, event.index2) end,
    }

  end


local app = App { song = renoise.song() }
